/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import com.sun.istack.NotNull;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.model.Branch;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.model.Repository;
import ru.ulstu.extractor.repository.RepositoryRepository;

import java.io.IOException;
import java.util.List;

@Service
public class IndexService {
    private final static Logger LOG = LoggerFactory.getLogger(IndexService.class);
    private final GitRepositoryService gitRepositoryService;
    private final RepositoryRepository repositoryRepository;
    private final BranchService branchService;

    public IndexService(GitRepositoryService gitRepositoryService,
                        RepositoryRepository repositoryRepository,
                        BranchService branchService) {
        this.gitRepositoryService = gitRepositoryService;
        this.repositoryRepository = repositoryRepository;
        this.branchService = branchService;
    }

    @Transactional
    public void index(@NotNull String repositoryUrl, @NotNull String branchName) throws GitAPIException, IOException {
        Repository repository = repositoryRepository.findByUrl(repositoryUrl);
        if (repository == null) {
            repository = repositoryRepository.save(new Repository(repositoryUrl));
        }
        Branch branch = branchService.findByRepositoryAndName(repository, branchName);
        if (branch == null) {
            branch = new Branch(repository, branchName);
        }
        List<Commit> commits = gitRepositoryService.getCommits(repositoryUrl, branchName);
        LOG.debug("{} commits loaded.", commits.size());
        branchService.save(branch, commits);
        LOG.debug("{} commits successfully saved. {} {}", commits.size(), repositoryUrl, branchName);
    }
}
